# Generated by Django 4.2.11 on 2024-06-06 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0011_authenticatormap_role_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticatormap',
            name='map_type',
            field=models.CharField(choices=[('allow', 'allow'), ('is_superuser', 'is_superuser'), ('role', 'role'), ('organization', 'organization'), ('team', 'team')], default='team', help_text='What will the map grant the user? System access (allow) a team or organization membership, the superuser flag or a role in the system', max_length=17),
        ),
    ]
