# Generated by Django 4.2.11 on 2024-06-06 22:46

import ansible_base.oauth2_provider.models.access_token
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_oauth2_provider', '0003_remove_oauth2application_logo_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oauth2accesstoken',
            name='scope',
            field=models.CharField(default='write', help_text="Allowed scopes, further restricts user's permissions. Must be a simple space-separated string with allowed scopes ['read', 'write'].", max_length=32, validators=[ansible_base.oauth2_provider.models.access_token.validate_scope]),
        ),
    ]
