# Generated by Django 4.2.8 on 2024-04-13 21:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dab_activitystream', '0002_alter_entry_created_by'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='entry',
            options={'ordering': ['id'], 'verbose_name_plural': 'Entries'},
        ),
    ]
