# Generated by Django 4.2.8 on 2024-03-11 18:57

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0003_alter_authenticatormap_authenticator'),
    ]

    operations = [
        migrations.RenameField(
            model_name='authenticator',
            old_name='created_on',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='authenticator',
            old_name='modified_on',
            new_name='modified',
        ),
        migrations.RenameField(
            model_name='authenticatormap',
            old_name='created_on',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='authenticatormap',
            old_name='modified_on',
            new_name='modified',
        ),
        migrations.AlterField(
            model_name='authenticator',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, help_text='The date/time this resource was created'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='authenticator',
            name='modified',
            field=models.DateTimeField(auto_now=True, help_text='The date/time this resource was created'),
        ),
        migrations.AlterField(
            model_name='authenticatormap',
            name='created',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, help_text='The date/time this resource was created'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='authenticatormap',
            name='modified',
            field=models.DateTimeField(auto_now=True, help_text='The date/time this resource was created'),
        ),
    ]
