# Generated by Django 4.2.8 on 2024-03-18 14:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, help_text='The date/time this resource was created')),
                ('object_id', models.TextField(blank=True, null=True)),
                ('operation', models.CharField(choices=[('create', 'Entity created'), ('update', 'Entity updated'), ('delete', 'Entity deleted'), ('associate', 'Entity was associated with another entity'), ('disassociate', 'Entity was disassociated with another entity')], max_length=12)),
                ('changes', models.JSONField(blank=True, null=True)),
                ('related_object_id', models.TextField(blank=True, null=True)),
                ('related_field_name', models.CharField(blank=True, max_length=64, null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='contenttypes.contenttype')),
                ('created_by', models.ForeignKey(default=None, editable=False, help_text='The user who created this resource', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='%(app_label)s_%(class)s_created+', to=settings.AUTH_USER_MODEL)),
                ('related_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='related_content_type', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name_plural': 'Entries',
            },
        ),
    ]
