# Generated by Django 4.2.11 on 2024-09-10 14:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0013_alter_authenticator_order'),
    ]

    operations = [
        migrations.AddField(
            model_name='authenticator',
            name='auto_migrate_users_to',
            field=models.ForeignKey(help_text='Automatically move users from this authenticator to the target authenticator when a matching user logs in via the target authenticator. For this to work, the field used for the user ID on both authenticators needs to have the same value. This should only be used when migrating users between two authentication mechanisms that share the same user database (such as when both IDPs share the same LDAP user directory).', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='auto_migrate_users_from', to='dab_authentication.authenticator'),
        ),
    ]
