# Generated by Django 4.2.6 on 2024-02-14 16:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0002_alter_authenticator_users'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticatormap',
            name='authenticator',
            field=models.ForeignKey(help_text='The authenticator this mapping belongs to', on_delete=django.db.models.deletion.CASCADE, related_name='authenticator_maps', to='dab_authentication.authenticator'),
        ),
    ]
