from django.apps import AppConfig


class RestPaginationConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'ansible_base.rest_pagination'
    label = 'dab_rest_pagination'
