# Generated by Django 4.2.17 on 2025-02-04 20:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0016_alter_authenticatoruser_access_allowed_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticator',
            name='slug',
            field=models.SlugField(default=None, editable=False, help_text='An immutable identifier for the authenticator; used to generate the sso uri for sso authenticator types', max_length=1024, unique=True),
        ),
    ]
