# Generated by Django 4.2.16 on 2024-11-22 21:25

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('dab_oauth2_provider', '0006_alter_oauth2accesstoken_created_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='oauth2accesstoken',
            name='application',
            field=models.ForeignKey(blank=True, help_text='The related application. If None, this is a user token instead of an application token.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='access_tokens', to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL),
        ),
        migrations.AlterField(
            model_name='oauth2accesstoken',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A description for this token.'),
        ),
        migrations.AlterField(
            model_name='oauth2accesstoken',
            name='last_used',
            field=models.DateTimeField(default=None, editable=False, help_text='A timestamp of when this token was last used.', null=True),
        ),
        migrations.AlterField(
            model_name='oauth2accesstoken',
            name='token',
            field=models.CharField(help_text='The generated token value.', max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name='oauth2application',
            name='description',
            field=models.TextField(blank=True, default='', help_text='A description of this application.'),
        ),
        migrations.AlterField(
            model_name='oauth2application',
            name='user',
            field=models.ForeignKey(blank=True, help_text='The user who created the application.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='applications', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='oauth2refreshtoken',
            name='token',
            field=models.CharField(help_text='The refresh token value.', max_length=255),
        ),
    ]
