# Generated by Django 4.2.8 on 2024-04-10 19:57

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('dab_authentication', '0005_alter_authenticator_created_by_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='authenticatoruser',
            unique_together={('provider', 'uid', 'user')},
        ),
    ]
