# Generated by Django 4.2.8 on 2024-04-10 20:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0007_remove_authenticator_users_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='authenticator',
            name='users_unique',
        ),
    ]
