# Generated by Django 4.2.11 on 2024-07-31 20:25

import ansible_base.authentication.models.authenticator
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0012_alter_authenticatormap_map_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticator',
            name='order',
            field=models.IntegerField(default=ansible_base.authentication.models.authenticator.get_next_authenticator_order, help_text='The order in which an authenticator will be tried. This only pertains to username/password authenticators'),
        ),
    ]
