# Generated by Django 4.2.17 on 2025-02-25 14:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_oauth2_provider', '0007_alter_oauth2accesstoken_application_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauth2application',
            name='app_url',
            field=models.URLField(blank=True, default=None, help_text='The URL of this application.', null=True),
        ),
    ]
