# Generated by Django 4.2.16 on 2024-11-22 21:25

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('dab_authentication', '0015_alter_authenticator_category_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authenticatoruser',
            name='access_allowed',
            field=models.BooleanField(default=None, help_text='Tracks if this user was allowed access to the system from the authenticator maps.', null=True),
        ),
        migrations.AlterField(
            model_name='authenticatoruser',
            name='claims',
            field=models.JSONField(blank=True, default=dict, help_text='The claims for the user as generated by the authenticator maps on their last login.'),
        ),
        migrations.AlterField(
            model_name='authenticatoruser',
            name='last_login_map_results',
            field=models.JSONField(blank=True, default=list, help_text='A data structure indicating how the authenticator maps were evaluated for the last login attempt.'),
        ),
        migrations.AlterField(
            model_name='authenticatoruser',
            name='provider',
            field=models.ForeignKey(help_text='The provider this user authenticated from.', on_delete=django.db.models.deletion.PROTECT, related_name='authenticator_providers', to='dab_authentication.authenticator', to_field='slug'),
        ),
        migrations.AlterField(
            model_name='authenticatoruser',
            name='user',
            field=models.ForeignKey(help_text='The local DB user related to this authenticator user.', on_delete=django.db.models.deletion.CASCADE, related_name='authenticator_users', to=settings.AUTH_USER_MODEL),
        ),
    ]
