# Generated by Django 4.2.16 on 2024-11-21 11:14

import ansible_base.resource_registry.models.service_identifier
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_resource_registry', '0005_resource_is_partially_migrated_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='resource',
            name='service_id',
            field=models.UUIDField(default=ansible_base.resource_registry.models.service_identifier.service_id, help_text='The ID of the service responsible for managing this resource.'),
        ),
    ]
