# Generated by Django 4.2.6 on 2024-03-02 14:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_resource_registry', '0002_remove_resource_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='resource',
            name='object_id',
            field=models.TextField(db_index=True),
        ),
    ]
