# Generated by Django 4.2.16 on 2024-11-22 21:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('dab_activitystream', '0004_alter_entry_created_alter_entry_created_by'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entry',
            name='changes',
            field=models.JSONField(blank=True, help_text='The changes to the system recorded by this entry.', null=True),
        ),
        migrations.AlterField(
            model_name='entry',
            name='content_type',
            field=models.ForeignKey(help_text='The content type which was changed in this entry.', on_delete=django.db.models.deletion.DO_NOTHING, to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='entry',
            name='object_id',
            field=models.TextField(blank=True, help_text='The object id which was modified in this entry.', null=True),
        ),
        migrations.AlterField(
            model_name='entry',
            name='operation',
            field=models.CharField(choices=[('create', 'Entity created'), ('update', 'Entity updated'), ('delete', 'Entity deleted'), ('associate', 'Entity was associated with another entity'), ('disassociate', 'Entity was disassociated with another entity')], help_text='The type of change recorded by the entry (i.e. create/update/delete/etc).', max_length=12),
        ),
        migrations.AlterField(
            model_name='entry',
            name='related_content_type',
            field=models.ForeignKey(blank=True, help_text='The content type if this entry is recording an many to many (M2M) change.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='related_content_type', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='entry',
            name='related_field_name',
            field=models.CharField(blank=True, help_text='The related field name if this entry is for an many to many (M2M) relationship.', max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='entry',
            name='related_object_id',
            field=models.TextField(blank=True, help_text='The object id of the related model if this entry is for an many to many (M2M) change.', null=True),
        ),
    ]
