# Generated by Django 4.2.11 on 2024-05-28 08:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dab_authentication', '0010_alter_authenticatoruser_unique_together'),
    ]

    operations = [
        migrations.AddField(
            model_name='authenticatormap',
            name='role',
            field=models.CharField(blank=True, default=None, help_text='The role this map will grant the authenticating user to the targeted object', max_length=512, null=True),
        ),
        migrations.AlterField(
            model_name='authenticatormap',
            name='map_type',
            field=models.CharField(choices=[('allow', 'allow'), ('is_superuser', 'is_superuser'), ('role', 'role'), ('organization', 'organization'), ('team', 'team')], default='team', help_text='What does the map work on, a team, organization, a user flag or is this an allow rule', max_length=17),
        ),
    ]
